/* file getxext.cpp
 *
 *  Read a resource file and handle the stuffs for internationalized
 *  printing.
 *  if the wanted text is not available in the internationalized
 *  version return the default (english).
 *
 */
#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

static char * knownPathes[] = {
   ".",
   ".CanonUtil",
   "/usr/lib/CanonUtil",
   "/usr/local/lib/CanonUtil",
   "/opt/CanonUtil",
   NULL,
};

static void addFallback(int entries);

/* place for fallback strings */
static char **resources;

static char *fallbackResources[] = {
   ".About: About",
   ".Close: Close",
   ".Cancel: Cancel",
   ".Quit: Exit",
   ".OK: OK",
   ".CanonBjCUtility: Canon BJC Utility",
   ".Configure: Preference",
   ".Help: Help",
   ".print_nozzle_check: print nozzle check",
   ".clean_black_head: clean black head",
   ".clean_both_head: clean all head",
   ".auto_adjust_head: auto adjust head",
   ".power_off: power off",
   ".reset_printer: reset printer",
   ".auto_power_off_disabled: disable auto power off",
   ".auto_power_off_enabled:  enable  auto power off",
   ".auto_power_on_disabled:  disable auto power on",
   ".auto_power_on_enabled:   enable  auto power on",
   ".msgSelectionTime: Please enter a time\nbetween 1 and 60 minutes.",
   ".SelectionTime: Time (1 - 60 min.)",
   ".PrinterSelection: Printer Choice",
   ".DeviceSelection: Port Choice",
   ".BrowserSelection: Enter Browser",
   ".SelectionTime: Time (1 - 60 Min.)",
   ".statusButton: Printer State",
   ".shadow: Shadowed",
   ".normal: Normal",
   ".openGl: Metallic",
   ".WaitStatus:  Please wait,\nlook for printer state\n...",
   ".msgCommError: Communication problem\n"
                  "with the printer.\n\n"
                  "Possible cause:\n"
                  "- No rights for opening the printer port\n"
                  "- Printer not powered on\n"
                  "- Print occuring",
   ".noDevice: Please choose the printer port\nvia the \"Prefererence\" menu",
   ".noPrinter: Please choose the printer\nvia the \"Prefererence\" menu",
   ".status: Printer state :",
   ".printerError: Error !",
   ".headError: Print head not installed !",
   ".wasteError: Waste error !",
   ".printerOK: OK",
   ".printerBusy: Busy",
   "",
   ".installedCartridge: Installed head:",
   ".BK: Monochrome",
   ".PB: Monochrome pigment",
   ".CL: Color",
   ".LD: Low-density ink",
   ".VC: Integrated cartridge of 3 colors",
   ".VL: Low-density color inks",
   ".SC: Scanner Cartridge",
   ".NO: -",
   ".NS: Not supported cartridge",
   ".LS: Integrated cartridge of low-density ink",
   ".inkLevel: Ink level:",
   ".inkLow: low",
   ".black: - Black        :",
   ".cyan: - Cyan         :",
   ".magenta: - Magenta      :",
   ".yellow: - Yellow       :",
   ".lmagenta: - Light Magenta:",
   ".lcyan: - Light Cyan   :",
   NULL
};


/*******************************************************************/
/* Function freeArray                                              */
/*                                                                 */
/* free all memory allocated to an array of strings including      */
/* the array pointer                                               */
/*                                                                 */
/*******************************************************************/

static void freeArray(char **array, int no)
{
   int i;
   if ( array )
   {
      for ( i = 0; i < no; i++ )
      {
         if ( array[i] ) free(array[i]);
      }
      free(array);
   }
}

/*******************************************************************/
/* Function addFallback                                            */
/*                                                                 */
/* add the fallback resources to the resources read from file      */
/*                                                                 */
/*******************************************************************/

static void addFallback(int entries)
{
   int actEntries=0;
   int newEntries=0;

   if ( resources )
   {
      /* add the fallback resources to the array */
      while(resources[actEntries]) actEntries++;
      while(fallbackResources[newEntries]) newEntries++;
      resources = (char**)realloc(resources, sizeof(char**)*(actEntries + newEntries + 1));
      if ( ! resources )
      {
         resources = fallbackResources;
      }
      newEntries = 0;
      while(fallbackResources[newEntries])
         resources[actEntries++] = fallbackResources[newEntries++];
      resources[actEntries] = NULL;
      
   }
   else
   {
      resources = fallbackResources;
   }
}

/*******************************************************************/
/* Function skipSpace                                              */
/*                                                                 */
/* advance up to the first non space character and return a        */
/* pointer to this position                                        */
/*                                                                 */
/*******************************************************************/

static char *skipSpace(char *s)
{
   while ( *s && isspace(*s) )
      s++;
   return s;
}

/*******************************************************************/
/* Function terminateLine                                          */
/*                                                                 */
/* remove the trailing \n an return the line lenght                */
/*                                                                 */
/*******************************************************************/

static int terminateLine(char *s)
{
   int len = 0;
   while ( *s )
   {
      if ( *s == '\n' || *s == '\r' )
      {
         *s = '\0';
         break;
      }
      s++;
      len++;
   }
   return len;
}

/*******************************************************************/
/* Function readResourceFile                                       */
/*                                                                 */
/* read the external resource file and build our storage area      */
/*                                                                 */
/*******************************************************************/

static int readResourceFile(FILE *fp)
{
   char  buf[2048];
   char *s;
   int   len;
   int   oldLen  = 0;
   int   entries = 0;
   while(fgets(buf,sizeof(buf), fp))
   {
      len = terminateLine(buf);
      if ( entries && resources[entries-1][oldLen-1] != '\\' )
      {
         s = skipSpace(buf);
         /* correct the line lenght */
         len -= (s-buf);
         if ( *s == '!' || *s == '\0' )
            continue;
      }
      else
      {
         s = buf;
      }
   
      if ( *s == '\0' )
         continue;
      if ( resources == NULL )
      {
         resources = (char **)malloc(sizeof(char*)*(entries+2));
         if ( resources == NULL )
         {
            entries = 0;
            break;
         }
         resources[1] = NULL;
      }
      else if ( resources[entries-1][oldLen-1] != '\\' )
      {
         resources = (char **)realloc(resources, sizeof(char*)*(entries+2));
         if ( resources == NULL )
         {
            entries = 0;
            break;
         }
         resources[entries+1] = NULL;
      }

      
      if ( entries && resources[entries-1][oldLen-1] == '\\' )
      {
         /* line continuation */
         resources[entries-1] = (char*)realloc(resources[entries-1], len+oldLen+1);
         if ( resources[entries-1] == NULL )
         {
            freeArray(resources, entries);
            resources = NULL;
            entries = 0;
            break;
         }
         strcpy(&resources[entries-1][oldLen-1], s);
         oldLen += len -1;
      }
      else
      {
         resources[entries] = strdup(s);
         if ( resources[entries] == NULL )
         {
            freeArray(resources, entries);
            resources = NULL;
            entries = 0;
            break;
         }
         oldLen = len;
         entries++;
      }
   }
   return entries;
}

/*******************************************************************/
/* Function checkRessourceString                                   */
/*                                                                 */
/* compare the key with the resource string, return NULL or the    */
/* message string found                                            */
/*                                                                 */
/*******************************************************************/

static char *checkRessourceString(char *res, char *key)
{
   char *msg = NULL;
   int len = strlen(key);
   if ( strncmp(res,key,len) == 0 )
   {
      if ( res[len] == ':' )
      {
         res += len;
         res++;
         while ( *res && (*res == ' ' || *res == '\t') ) res++;
         msg = res;
      }
   }
   return msg;
}

/*******************************************************************/
/* Function searchString                                           */
/*                                                                 */
/* search a string in our resource array and return it             */
/* if localized string is not found return the default string      */
/* if nothing is found return NULL                                 */
/* convert resource strings as made by Xt                          */
/*******************************************************************/

char *searchString(char *key)
{
   char **msg = resources;
   char *defaultMsg   = NULL;
   char  *s;
   
   while( *msg && defaultMsg == NULL )
   {
      /* go to the first . or * character */
      s = *msg;
      while ( *s && ( *s != '.' && *s != '*' ) ) s++;
      if ( *s ) s++;
      if ( *s )
      {
         /* check for the default string */
         if ( ! defaultMsg )
            defaultMsg = checkRessourceString(s, key);
      }
      msg++;
   }
   return defaultMsg;
}


/*******************************************************************/
/* Function correctResources                                       */
/*                                                                 */
/* convert escaped characters to the target character fot each     */
/* resource read                                                   */
/*                                                                 */
/*******************************************************************/

void correctResources(int entries)
{
   int i;
   char *s, *t;
   char **msg = resources;

   for ( i = 0; i < entries; i++ )
   {
      /* replace '\*' with the correct value */
      s = msg[i];
      t = msg[i];
      while ( *s )
      {
         switch(*s)
         {
            case '\\':
               s++;
               if ( *s == 'n' )
               {
                  *t++ = '\n';
                  s++;
               }
               else
               {
                  *t++ = *s++;
               }
            break;
            default:
               *t++ = *s++;
         }
      }
      *t = '\0';
   }
}

/*******************************************************************/
/* Function sprintText                                             */
/*                                                                 */
/* search the string referenced by lang and key, then save it into */
/* the external array *out                                         */
/*                                                                 */
/*******************************************************************/

char *getText(char *key)
{
   char *msg;
   msg = searchString(key);
   if ( msg )
   {
      return msg;
   }
   else
   {
      return key;
   }
}

/*******************************************************************/
/* Function initResource                                           */
/*                                                                 */
/* Read textes from the given file                                 */
/*                                                                 */
/*******************************************************************/
extern int errno;
int initResource(char *file, char *lang)
{
   char  *resDir = NULL;
   char  *path;
   int    entries;
   int    i = 0;
   FILE *fp;

   /* search for the file CanonUtilFltk_<lang>.res */
   for ( i = 0; (resDir = knownPathes[i]);i++ )
   {
      if ( (path = (char*) calloc(strlen(resDir)+strlen(file)+2+strlen(file)+2+strlen(lang)+10,1)) != NULL )
      {
         sprintf(path, "%s/%s_%s.res", resDir, file,lang);
         if ( (fp = fopen(path, "r")) )
         {
            free(path);
            /* read the file */
            entries = readResourceFile(fp);
            fclose(fp);
            if ( entries )
              correctResources(entries);
            addFallback(entries);
            return 1;
         }
        free(path);
      }
      resDir++;
   }
   addFallback(0);
   return 0;
}
#if defined(__cplusplus) || defined(c_plusplus)
}
#endif
